<?php
//PSR-0 namespace
namespace Module\Blog\Admin;
//We need Model to work with the database
use Model\Model;
//JSON\Admin is the parent class
use Module\JSON\Admin;

require_once(dirname(dirname(dirname(dirname(__FILE__)))) . '/Utils/functions.php');
class JSON extends Admin {
	public function __construct() {
		//Set permission for Read action
		$this->permission = "Manage blog";
		parent::__construct();
	}
	public function get() {
		//Check if we need a specific post (for edit it's data) or we need a multiple posts (for the data table)
		if(!arrayKeyExists('id', $_REQUEST)) {
			//$posts is the database model
			$posts = new Model('posts');
			//If we selected a number of posts per page then set $itemsPerPage as that number, else set it to 10 posts per page
			$itemsPerPage = (arrayKeyExists('length', $_REQUEST))?$_REQUEST['length']:10;
			//If we have clicked on a page number then set the starting point for the query's limit
			$limit = ((arrayKeyExists('start', $_REQUEST))?$_REQUEST['start']:0) . ', ' . $itemsPerPage;
			//Count the total posts
			$countTotal = $posts->countItems();
			//Build the filters parameter
			if(arrayKeyExists('filters', $_REQUEST)) {
				foreach($_REQUEST['filters'] AS $key => $value) {
					//Use wildcard if we search by the title
					if(in_array($key, array('title'))) $posts->$key = array('%' . $value . '%', ' LIKE ');
					//Use exact match for every other filter
					else $posts->$key = $value;
				}
			}
			//Count the filtered posts
			$countFiltered = $posts->countItems();
			//Set the limit (starting point, posts per page)
			$posts->limit($limit);
			//Order posts by their updated value descending
			$posts->order('date_updated DESC');
			//Get the filtered & limited posts
			$posts = $posts->get('AND');
			$aaData = array();
			foreach($posts AS $post) {
				//We don't need content for every post because we don't show the value in the data table and will create a huge json if we don't unset it
				unset($post->content);
				//Set the thumbnail filename for every post
				$post->image = str_replace('.jpg', '-360x220.jpg', $post->image);
				//Add the object to the returning array
				$aaData[] = $post;
			}
			//Build the returning array
			$responseArray = array('sEcho' => $_REQUEST['secho'],
				'iTotalRecords'        => $countTotal,
				'iTotalDisplayRecords' => $countFiltered,
				'aaData'               => $aaData
			);
			//Encode the array to json
			$response = json_encode($responseArray);
			//If we have a callback parameter then add it to the response
			if(arrayKeyExists('callback', $_GET)) $response = $_GET['callback'] . '(' . $response . ')';
			echo $response;
		}
		else {
			//We searched a post by it's id
			$posts = new Model('posts');
			$posts = $posts->getOneResult('id', $_REQUEST['id']);
			//Set the thumbnail filename for the post
			$posts->image = str_replace('.jpg', '-360x220.jpg', $posts->image);
			echo json_encode($posts);
		}
	}
}