<?php
namespace Module\Products\Admin;
use Controller\AdminAct;
use Controller\AdminController;
use Model\Model;
use Module\Products\Setup;
use \PHPThumb\GD;
use Utils\Util;

require_once(dirname(dirname(dirname(dirname(__FILE__)))) . '/Utils/functions.php');

class Act extends AdminAct {
	private $pictures = array();
	private $categories = array();
	public function __construct($id) {
		//Permission needed for editing this type of entity
		$this->permission = 'Manage shop';
		//Table name
		$this->entity = new Model('products');
        $act = false;
        if(!file_exists(_APP_DIR_ . 'assets/img/products/')) mkdir(_APP_DIR_ . 'assets/img/products/');
        if ($this->hasAccess()) {
            if ($id) $this->fields['id'] = $id;
            if (strtolower($_SERVER['REQUEST_METHOD']) == 'delete') {
                if ($id) {
                    $act = $this->delete();
                }
                else $this->sendStatus(false, __('No ID set'));
            }
            else {
                $method = 'patch';
                if (strtolower($_SERVER['REQUEST_METHOD']) == 'patch') {
                    if ($id) parse_str(file_get_contents('php://input'), $_PATCH);
                    else $this->sendStatus(false, __('No ID set'));
                }
                else $method = 'create';
                foreach ($method == 'patch' ? $_PATCH : $_POST AS $key => $value) {
                    if(in_array($key, array('id', 'name', 'sku', 'description', 'price', 'stock', 'sort_order', 'url', 'created', 'updated'))) $this->fields[$key] = $value;
                    elseif($key == 'pictures') $this->pictures = $value;
                    elseif($key == 'categories') $this->categories = $value;
                }
                try {
                    $act = call_user_func_array(array($this, $method), array());
                }
                catch (\Exception $e) {
                    $this->sendStatus(false, $e->getMessage());
                }
            }
        }
        $this->sendStatus($act);
	}

    public function create() {
        $sortOrder = 1;
        $products = new Model('products');
        $products->addCustomField('MAX(sort_order) AS last_sort_order');
        $products->addCustomJoin(array('category_products' => array('column' => 'id', 'reference' => 'product', 'columns' => [], 'joinType' => 'INNER')));
        $products = $products->get();
        if(count($products)) $sortOrder = $products[0]->last_sort_order;
        foreach ($this->fields AS $key => $value) $this->entity->$key = $value;
        $this->entity->sort_order = $sortOrder;
        $act = $this->entity->create();
        if(is_a($act, 'Model\Model')) {
            $this->fields['id'] = $act->id;
            $this->setupProductUrl()->setCategoriesAndPhotos();
        }
        return $act;
    }

    public function patch() {
        foreach ($this->fields AS $key => $value) $this->entity->$key = $value;
        $act = $this->entity->update();
        if(is_a($act, 'Model\Model')) $this->setupProductUrl(true)->setCategoriesAndPhotos();
        return $act;
    }

    public function delete() {
        $this->entity->id = $this->fields['id'];
        //We have to delete the URL for the product
        $oldProductValues = new Model('products');
        $oldProductValues = $oldProductValues->getOneResult('id', $this->fields['id']);
        $mR = new Model('module_routes');
        $mR = $mR->getOneResult('url', $oldProductValues->url);
        if ($mR && $mR->modules->name == 'Products') {
            $mR->delete();
        }
        return $this->entity->delete();
    }

    private function setupProductUrl($isUpdate = false) {
        //If it is an update, delete the old url for the product if it's changed
        if($isUpdate) {
            $oldProductValues = new Model('products');
            $oldProductValues = $oldProductValues->getOneResult('id', $this->fields['id']);
            if($oldProductValues->url != $this->fields['url']) {
                $mR = new Model('module_routes');
                $mR = $mR->getOneResult('url', $oldProductValues->url);
                if ($mR && $mR->modules->name == 'Products') {
                    $mR->delete();
                }
            }
        }
        $mR = new Model('module_routes');
        $mR = $mR->getOneResult('url', $this->fields['url']);
        if(!$mR) {
            $setup = new Setup();
            $setup->registerFrontendUrl(array('url' => $this->fields['url'], 'type' => 0, 'mustBeLoggedIn' => 0, 'menu_position' => 0));
        }
        return $this;
    }

	private function setCategoriesAndPhotos() {
        $productPhotos = new Model('product_photos');
        $category_products = new Model('category_products');
        //$entity is a Model if it's created; if updated, $entity is the ID of the entity
        //If we have an update for product we clear the categories and photos it has been before update
        if(!is_a($this->entity, 'Model\Model')) {
            $category_products->product = $this->fields['id'];
            $category_products->delete();
            $productPhotos->product = $this->fields['id'];
            $productPhotos->delete();
        }
        //We set the categories selected for the product
        $category_products->clear();
        foreach($this->categories AS $idCategory) {
            $category_products->category = $idCategory;
            $category_products->product = $this->fields['id'];
            $category_products->create();
            $category_products->clear();
        }
        foreach($this->pictures AS $picture) {
            if(!empty(trim(strip_tags($picture)))) {
                $filename = strip_tags($picture);
                $upload_dir = _APP_DIR_ . 'uploads/';
                $uploaded_file = $upload_dir . $filename;
                $path_parts = pathinfo($uploaded_file);
                $fname = $path_parts['filename'];
                $extension = $path_parts['extension'];
                $target_dir = _APP_DIR_ . 'assets/img/products/';
                $destination_file = $target_dir . $filename;
                $thumb_file_name = 'thumb' . $path_parts['filename'] . '.jpg';
                $thumb_target_file = $upload_dir . $thumb_file_name;
                if(file_exists($thumb_target_file)) unlink($thumb_target_file);
                rename($uploaded_file, $destination_file);
                $thumb_file_name380 = $target_dir . $fname . '-380x460.' . $extension;
                $thumb_file_name760 = $target_dir . $fname . '-760x920.' . $extension;
                $thumb = new GD($destination_file);
                $thumb->resize(760, 920);
                $thumb->save($thumb_file_name760, $extension);
                $thumb->resize(380, 460);
                $thumb->save($thumb_file_name380, $extension);
                $productPhotos->product = $this->fields['id'];
                $productPhotos->filename = $filename;
                $productPhotos->create();
            }
        }
        return $this;
    }
}